"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkSymbolImportability = void 0;
const assertNever_1 = require("../utils/assertNever");
const concatArrays_1 = require("../utils/concatArrays");
const findExportableDeclaration_1 = require("../utils/findExportableDeclaration");
const getAccessOfJsDocs_1 = require("../utils/getAccessOfJsDocs");
const getJSDocTags_1 = require("../utils/getJSDocTags");
const isInPackage_1 = require("../utils/isInPackage");
const lookupPackageJson_1 = require("./lookupPackageJson");
function checkSymbolImportability(packageOptions, program, importerFilename, moduleSpecifier, exportedSymbol) {
    var _a;
    const rawDecl = (_a = exportedSymbol.declarations) === null || _a === void 0 ? void 0 : _a[0];
    if (!rawDecl) {
        return;
    }
    const decl = (0, findExportableDeclaration_1.findExportedDeclaration)(rawDecl);
    if (!decl) {
        return;
    }
    // If declaration is from external module, treat as importable
    if (program.isSourceFileFromExternalLibrary(decl.getSourceFile())) {
        return;
    }
    if (packageOptions.treatSelfReferenceAs === "external" && possibleSubpathImportFromPackage.test(moduleSpecifier)) {
        // Check whether this import is the result of a self-reference.
        const lookupResult = (0, lookupPackageJson_1.lookupPackageJson)(importerFilename);
        if (lookupResult !== null) {
            if (checkIfImportIsSelfReference(moduleSpecifier, lookupResult.packageJson.name)) {
                // This is a self-reference, so treat as external.
                return;
            }
        }
    }
    const checker = program.getTypeChecker();
    // found an export declaration
    const jsDocs = (0, concatArrays_1.concatArrays)(exportedSymbol.getJsDocTags(checker).map((tag) => {
        var _a;
        return ({
            name: tag.name,
            text: ((_a = tag.text) === null || _a === void 0 ? void 0 : _a[0].text) || "",
        });
    }), (0, getJSDocTags_1.getJSDocTags)(decl));
    if (!jsDocs) {
        switch (packageOptions.defaultImportability) {
            case "public":
                return;
            case "private":
                return "private";
            case "package": {
                const inPackage = (0, isInPackage_1.isInPackage)(importerFilename, decl.getSourceFile().fileName, packageOptions);
                return inPackage ? undefined : "package";
            }
        }
    }
    const access = (0, getAccessOfJsDocs_1.getAccessOfJsDocs)(jsDocs, packageOptions.defaultImportability);
    if (access === "public") {
        // no restriction
        return;
    }
    if (access === "private") {
        // no import of private stuff! (why is this exported?)
        return "private";
    }
    if (access !== "package") {
        (0, assertNever_1.assertNever)(access);
    }
    // for package-exports, check relation of this and that files
    const inPackage = (0, isInPackage_1.isInPackage)(importerFilename, decl.getSourceFile().fileName, packageOptions);
    return inPackage ? undefined : "package";
}
exports.checkSymbolImportability = checkSymbolImportability;
const possibleSubpathImportFromPackage = /^(?![./\\])([^/\\]*)(?:$|[/\\][^/\\])/;
const checkIfImportIsSelfReference = (moduleSpecifier, packageName) => {
    const importIsDefaultModule = moduleSpecifier === packageName;
    const importIsSubModule = moduleSpecifier.startsWith(`${packageName}/`);
    return importIsDefaultModule || importIsSubModule;
};
