import { TSESLint } from "@typescript-eslint/utils";
type MessageId = "no-program" | "package" | "package:reexport" | "private" | "private:reexport";
export type JSDocRuleOptions = {
    /**
     * Whether importing a package-private exports from `index.ts` in a subdirectory.
     */
    indexLoophole: boolean;
    /**
     * Whether importing a package-private exports in a directory from a file of same name.
     */
    filenameLoophole: boolean;
    /**
     * Whether packages importability is restricted to public exports only or not.
     */
    defaultImportability: "public" | "package" | "private";
    /**
     * Whether to treat self-reference as internal or external.
     * When `external`, imports using the self-referencing feature of Node.js are
     * treated as imports from external packages, meaning that they bypass
     * the importability check.
     */
    treatSelfReferenceAs: "internal" | "external";
};
declare const jsdocRule: Omit<TSESLint.RuleModule<MessageId, [Partial<JSDocRuleOptions>?]>, "docs">;
export default jsdocRule;
export declare function jsDocRuleDefaultOptions(options: Partial<JSDocRuleOptions> | undefined): JSDocRuleOptions;
